const BattleCalc = require('../BattleCalc');
const Balance = require('../../constants/Balance');

const AICalc = {
  SUPPORT_DECISION_LOW_HP_THRESHOLD: 80, // percent

  getBestMeleeTarget: function (actor, allEnemies) {
    if (allEnemies.length === 0) {
      return null;
    }

    var bestTarget = allEnemies[0];

    for (var i = 0; i < allEnemies.length; ++i) {
      var enemy = allEnemies[i];
      if (!enemy.inPlay) {
        continue;
      }

      var dist = BattleCalc.boardDistance(actor, enemy);
      var hpFactor = dist * enemy.hp;

      var currentBestDist = BattleCalc.boardDistance(actor, bestTarget);
      var currentBestHPFactor = currentBestDist * bestTarget.hp;

      if (hpFactor < currentBestHPFactor) {
        bestTarget = enemy;
      }
    }

    return bestTarget;
  },

  getBestMissileTarget: function (actor, candidates) {
    if (typeof candidates === 'object') {
      candidates = Object.values(candidates);
    }

    // figure out what my top range ability is
    let top_range = 0;
    for (const ability_handle of [
      'arrow_shot',
      'razor_shot',
      'void_spike',
      'infect_mind',
      'molotov_cocktail',
      'multi_shot',
    ]) {
      const ability = BattleCalc.getEquippedAbility(actor, ability_handle);
      if (ability?.range.max > top_range) {
        top_range = ability.range.max;
      }
    }

    const missile_target = getLowestHealthCandidateInRange(top_range);
    return missile_target;

    function getLowestHealthCandidateInRange(range) {
      let lowest_health = 99999999;
      let result;
      // find candidate lowest health within range (if nx, increment range)
      for (let r = 1; r <= range; r++) {
        for (const candidate of candidates) {
          if (BattleCalc.boardDistance(actor, candidate) <= r) {
            if (candidate.hp < lowest_health) {
              lowest_health = candidate.hp;
              result = candidate;
            }
          }
        }
      }

      return result
        ? result
        : range < 50
        ? getLowestHealthCandidateInRange(range + 1)
        : AICalc.getBestMeleeTarget(actor, candidates);
    }
  },

  defineEnemyCluster: function (cache, target, candidates) {
    cache.splice(0, cache.length);

    for (var i = 0; i < candidates.length; ++i) {
      var enemy = candidates[i];
      if (BattleCalc.boardDistance(enemy, target) < 3) {
        cache.push(enemy);
      }
    }

    return cache;
  },

  getNearbyUnits: function (subject, candidates, defineNearby) {
    var nearbyUnits = [];
    for (var i = 0; i < candidates.length; ++i) {
      var unit = candidates[i];
      if (BattleCalc.boardDistance(subject, unit) <= defineNearby) {
        nearbyUnits.push(unit);
      }
    }

    return nearbyUnits;
  },

  getBestSummonTile(actor, battleState) {
    var tile = null;
    var candidateTiles = [];
    var attempts = 0;

    var distFromActor = 1;
    do {
      for (var tid in battleState.allTiles) {
        var tile = battleState.allTiles[tid];
        if (
          BattleCalc.boardDistance(tile, actor) <= distFromActor &&
          !tile.occupied
        ) {
          candidateTiles.push(tile);
        }
      }
      ++distFromActor;
    } while (
      distFromActor <= Balance.CONJURATION_RANGE &&
      candidateTiles.length === 0 &&
      ++attempts < 1000
    );

    if (candidateTiles.length === 0) {
      return null;
    } else {
      return candidateTiles[Math.floor(Math.random() * candidateTiles.length)];
    }
  },
};
module.exports = AICalc;
