/**
 * ...
 * @author Robert Lombardo, Wakefield Studios ( rob@wakefield-studios.com )
 */
const Q_how_many_nearby_enemies_BASE = require('../questions/Q_how_many_nearby_enemies_BASE');
const Q_enemy_cluster = require('./Q_enemy_cluster');
const AttackerBuffDecision = require('../buff/AttackerBuffDecision');
const BattleCalc = require('../../BattleCalc');

const DPSRangedDecision = function (actor, battleState) {
  const node = new Q_how_many_nearby_enemies_BASE(actor, battleState);

  // branch indexes
  // const for zero is taken by ... well, zero (in the parent class)
  node.MORE_THAN_TWO = 1;

  node.branches[node.MORE_THAN_TWO] = new Q_enemy_cluster(actor, battleState);

  // we pass this a reference to the "enemy cluster" node, as an alternate in case no AOE abilities are equipped
  node.branches[node.ZERO] = new AttackerBuffDecision(
    actor,
    battleState,
    node.branches[node.MORE_THAN_TWO]
  );

  node.arrowShot = BattleCalc.getEquippedAbility(actor, 'arrow_shot');
  node.voidSpike = BattleCalc.getEquippedAbility(actor, 'void_spike');

  node.query = function () {
    const getBestMissileAttackRange = () => {
      var bestDamage = 0;

      if (this.arrowShot) {
        bestDamage = this.arrowShot.damage;
      }

      if (this.voidSpike && this.voidSpike.damage > bestDamage) {
        return this.voidSpike.range.max;
      } else if (this.arrowShot) {
        return this.arrowShot.range.max;
      } else {
        return 1;
      }
    };

    const numNearby = this.calculateNearbyEnemies(getBestMissileAttackRange());

    if (numNearby < 3) {
      // give the hero a chance to buff
      return this.branches[this.ZERO].query();
    } else {
      return this.branches[this.MORE_THAN_TWO].query();
    }
  };

  return node;
};
module.exports = DPSRangedDecision;
