/**
 * ...
 * @author Robert Lombardo, Wakefield Studios ( rob@wakefield-studios.com )
 */
const YesNoNode = require('../YesNoNode');
const BattleCalc = require('../../BattleCalc');
const Q_which_aoe = require('./Q_which_aoe');
const Q_better_to_melee = require('./Q_better_to_melee');
const AICalc = require('../AICalc');

const Q_enemy_cluster = function (actor, battleState) {
  const node = new YesNoNode(actor, battleState);

  node.arrowShot = BattleCalc.getEquippedAbility(actor, 'arrow_shot');
  node.voidSpike = BattleCalc.getEquippedAbility(actor, 'void_spike');

  node.branches[node.YES] = new Q_which_aoe(actor, battleState);
  node.branches[node.NO] = new Q_better_to_melee(actor, battleState);

  node.query = function () {
    const bestTarget = AICalc.getBestMissileTarget(
      this.actor,
      this.battleState.getEnemiesOfUnit(this.actor)
    );
    if (!bestTarget) {
      return null;
    }

    var enemyCluster = AICalc.defineEnemyCluster(
      [],
      bestTarget,
      this.battleState.getEnemiesOfUnit(this.actor)
    );

    if (enemyCluster.length > 2 || (!this.arrowShot && !this.voidSpike)) {
      return this.branches[this.YES].query(bestTarget);
    } else {
      return this.branches[this.NO].query(bestTarget);
    }
  };

  return node;
};
module.exports = Q_enemy_cluster;
