/**
 * ...
 * @author Robert Lombardo, Wakefield Studios ( rob@wakefield-studios.com )
 */
const TreeNode = require('../TreeNode');
const UseAbility = require('../UseAbility');
const BattleCalc = require('../../BattleCalc');
const AICalc = require('../AICalc');
const Q_target_high_hp = require('./Q_target_high_hp');

const Q_which_aoe = function (actor, battleState) {
  const node = new TreeNode(actor, battleState);

  // branch indexes
  node.MULTI_SHOT = 0;
  node.MOLOTOV = 1;

  node.multiShot = BattleCalc.getEquippedAbility(actor, 'multi_shot');
  node.molotov = BattleCalc.getEquippedAbility(actor, 'molotov_cocktail');

  const altNode = new Q_target_high_hp(actor, battleState);

  if (node.multiShot) {
    node.branches[node.MULTI_SHOT] = new UseAbility(actor, 'multi_shot');
  } else {
    node.branches[node.MULTI_SHOT] = altNode;
  }

  if (node.molotov) {
    node.branches[node.MOLOTOV] = new UseAbility(actor, 'molotov_cocktail');
  } else {
    node.branches[node.MOLOTOV] = altNode;
  }

  node.query = function (target) {
    if (this.multiShot) {
      return this.branches[this.MULTI_SHOT].query(target);
    }
    // don't aoe if there is more than 1 ally near the target
    else if (
      this.molotov &&
      AICalc.getNearbyUnits(
        target,
        BattleCalc.getAlliesOfUnit(this.actor, this.battleState, true),
        2
      ).length < 2
    ) {
      return this.branches[this.MOLOTOV].query(target);
    } else {
      return this.branches[this.MULTI_SHOT].query(target);
    }
  };

  return node;
};
module.exports = Q_which_aoe;
