const MoveDecision = require('./MoveDecision');
const BattleCalc = require('../../BattleCalc');

const MoveTowardsDecision = function(actor, battleState) {
  var node = new MoveDecision(actor, battleState);

  node.maxJump = node.jumpRange;

  node.getBestTileAtMaxJump = function(ultimateTarget) {
    var candidateTiles = [];

    for (var tid in this.battleState.allTiles) {
      var tile = this.battleState.allTiles[ tid ];
      if (BattleCalc.boardDistance(actor,tile) === this.maxJump
				&&	BattleCalc.pieceIsBetweenPair(tile,[this.actor,ultimateTarget])
				&&	!tile.occupied)
      {
        candidateTiles.push(tile);
      }
    }

    if (candidateTiles.length === 0) {
      return ultimateTarget;
    }

    return candidateTiles[ Math.floor(Math.random()*candidateTiles.length) ];
  };

  node.getRelevantTile = function(target) {
    if (!target) {
      return null;
    }

    var leap = BattleCalc.getEquippedAbility(this.actor,'leap');
    var teleport = BattleCalc.getEquippedAbility(this.actor,'teleport');

    if ((leap || teleport)
			&&	(target.unit || target.occupied || BattleCalc.boardDistance(this.actor,target) > this.maxJump))
    {
      target = this.getBestTileAtMaxJump(target);
    }

    if (!target) {
      return null;
    }

    if (target.unit) {
      return this.battleState.getTileAtCoords(target.x, target.y);
    } else {
      return target;
    }
  };

  return node;
};
module.exports = MoveTowardsDecision;
