/**
 * ...
 * @author Robert Lombardo, Wakefield Studios ( rob@wakefield-studios.com )
 */
const YesNoNode = require('../YesNoNode');
const BattleCalc = require('../../BattleCalc');
const { logError } = require('../../logger');

const Q_buff_ally = function (buffHandle, actor, battleState) {
  const node = new YesNoNode(actor, battleState);

  node.buffAbility = BattleCalc.getEquippedAbility(actor, buffHandle);

  switch (buffHandle) {
    case 'bless':
      node.buffCondition = 'blessed';
      break;
    case 'summon_option':
      node.buffCondition = 'has_option';
      break;
    case 'poison_weapon':
      node.buffCondition = 'poisoner';
      break;
    default:
      logError(err, {
        module: 'Q_buff_ally',
        func: 'constructor',
        buffHandle,
        actor,
        battleState,
      });
      break;
  }

  if (!node.buffAbility) {
    node.branches[node.YES] = node.branches[node.NO];
  }

  node.getUnitWithoutBuff = function () {
    for (var prop in this.battleState.allUnits) {
      var unit = this.battleState.allUnits[prop];

      if (
        unit.team === this.actor.team &&
        !BattleCalc.getCondition(unit, this.buffCondition)
      ) {
        return unit;
      }
    }

    return null;
  };

  node.query = function () {
    if (!this.buffAbility) {
      return this.branches[this.NO].query();
    }

    var unitWithoutBuff = this.getUnitWithoutBuff();

    if (unitWithoutBuff) {
      return this.branches[this.YES].query(unitWithoutBuff);
    } else {
      return this.branches[this.NO].query();
    }
  };

  return node;
};
module.exports = Q_buff_ally;
