const TreeNode = require('../TreeNode');
const BattleCalc = require('../../BattleCalc');
const UseAbility = require('../UseAbility');
const AICalc = require('../AICalc');

const Q_which_summon = function (actor, battleState) {
  const node = new TreeNode(actor, battleState);

  node.FAERIE = 0;
  node.DWARF = 1;
  node.ELF = 2;

  node.summonFaerie = BattleCalc.getEquippedAbility(actor, 'summon_faerie');
  node.summonDwarf = BattleCalc.getEquippedAbility(actor, 'summon_dwarf');
  node.summonElf = BattleCalc.getEquippedAbility(actor, 'summon_elf');

  if (node.summonFaerie) {
    node.branches[node.FAERIE] = new UseAbility(actor, 'summon_faerie');
  }

  if (node.summonDwarf) {
    node.branches[node.DWARF] = new UseAbility(actor, 'summon_dwarf');
  }

  if (node.summonElf) {
    node.branches[node.ELF] = new UseAbility(actor, 'summon_elf');
  }

  node.getNumFriendlySummonedUnits = function () {
    var faeries = 0;
    var dwarves = 0;
    var elves = 0;
    const allies = BattleCalc.getAlliesOfUnit(
      this.actor,
      this.battleState,
      false
    );

    for (var i = 0; i < allies.length; ++i) {
      var unit = allies[i];
      if (unit.handle === 'faerie') {
        ++faeries;
      } else if (unit.handle === 'dwarf') {
        ++dwarves;
      } else if (unit.handle === 'elf') {
        ++elves;
      }
    }

    return {
      faeries,
      dwarves,
      elves,
    };
  };

  node.query = function () {
    const num = this.getNumFriendlySummonedUnits();
    const summonTile = AICalc.getBestSummonTile(actor, battleState);

    if (
      this.summonFaerie &&
      (num.faeries <= num.dwarves || !this.summonDwarf) &&
      (num.faeries <= num.elves || !this.summonElf)
    ) {
      return this.branches[this.FAERIE].query(summonTile);
    } else if (
      this.summonDwarf &&
      (num.dwarves <= num.faeries || !this.summonFaerie) &&
      (num.dwarves <= num.elves || !this.summonElf)
    ) {
      return this.branches[this.DWARF].query(summonTile);
    } else if (this.summonElf) {
      return this.branches[this.ELF].query(summonTile);
    } else {
      return null;
    }
  };

  return node;
};
module.exports = Q_which_summon;
