const HexTile = require('./HexTile');
const DoorsFactory = require('./DoorsFactory');
const PortalFactory = require('./PortalFactory');
const StairsFactory = require('./StairsFactory');
const WallsFactory = require('./WallsFactory');

const HexRoom = function(entrance='SOUTH', exits=['NORTH','EAST','WEST'], cavernsLevel=1) {
  const columns = 19;
  const rows = 23;

  const outOfBounds = function(x, y) {
    return x+y < 5
      ||	y-x > 17
      ||	x-y > 13
      ||	x+y > 35;
  };

  const tiles = [];
  let x, y;

  for (x = 0; x < columns; x += 2) {
    for (y = 1; y < rows; y += 2) {
      if (!outOfBounds(x, y)) {
        tiles.push(HexTile(x,y));
      }
    }
  }

  for (x = 1; x < columns; x += 2) {
    for (y = 0; y < rows; y += 2) {
      if (!outOfBounds(x, y)) {
        tiles.push(HexTile(x,y));
      }
    }
  }

  const doors = DoorsFactory(tiles,entrance,exits).concat(
    StairsFactory(tiles,entrance,cavernsLevel, 3),
  );

  return {
    tiles,
    doors,
    walls: WallsFactory(tiles, doors),
    portal: cavernsLevel > 1 && PortalFactory(tiles),
  };
};
module.exports = HexRoom;
