const ItemGenerator = require('../../../game_items/ItemGenerator.js');
const LightHelmet = require('../../../game_items/equipment/helmet/LightHelmet.js');
const LightTorsoe = require('../../../game_items/equipment/torsoe/MedTorsoe.js');
const LightBoots = require('../../../game_items/equipment/boots/LightBoots.js');
const Staff = require('../../../game_items/equipment/weapon/Staff.js');
const getMobEquipmenAssignmentFromBags = require('./getMobEquipmenAssignmentFromBags');

const kobold_cleric = (level) => {
  const handle = 'kobold_cleric';
  const AI_PRIORITIES = ['support', 'dps_close', 'tank', 'interference', 'dps_ranged'];
  const bags = [{
    items: [
      ItemGenerator.getRandomItem(LightHelmet, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(LightTorsoe, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(LightBoots, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(Staff, 0, Math.min(1,0.1+level/100)),
    ],
  }];

  return {
    handle,
    level,
    getAbilityTrees: ((bound_level) => ([{
      handle: 'kobold_cleric_abilities',
      abilities: {
        move: { handle: 'move', level: 1 },
        melee_attack: { handle: 'melee_attack', level: 1 },
        healing_prayer: { handle: 'healing_prayer', level: Math.min(10,Math.round(1+bound_level/2)) },
        mystic_focus: { handle: 'mystic_focus', level: Math.min(10,Math.round(1+bound_level/2)) },
        bless: { handle: 'bless', level: Math.min(10,Math.round(1+bound_level/2)) },
      },
    }])).bind(null, level),
    getBaseAttributes: ((bound_level) => ({
      learning: Math.round(2.6 + bound_level*0.7 + Math.random()*3),
      perception: Math.round(1.6 + bound_level*0.4 + Math.random()*3),
      mana: Math.round(2.2 + bound_level*0.6 + Math.random()*3),
      toughness: Math.round(1.7 + bound_level*0.45 + Math.random()*3),
      strength: Math.round(1.2 + bound_level*0.35 + Math.random()*3),
      willpower: Math.round(3.0 + bound_level*0.75 + Math.random()*3),
      dexterity: Math.round(1.9 + bound_level*0.6 + Math.random()*3),
    })).bind(null, level),
    getDefaultAIConfig: () => AI_PRIORITIES,
    xpValue: level * 750,
    bags,
    build: {
      ai: {
        priorities: AI_PRIORITIES,
        settings: {
          chanceToSkipRoles: true,
          autoMove: true,
          waitForAPMP: false,
        },
      },
      ai_unit: true,
      beast: false,
      equipment_assignment: getMobEquipmenAssignmentFromBags(bags),
      equipped_abilities: ['move', 'melee_attack', 'healing_prayer', 'mystic_focus', 'bless'],
      handle,
      humanoid: true,
      level,
      team: 'black',
      undead: false,
      unit: true,
    },
  };
};
module.exports = kobold_cleric;
