const ItemGenerator = require('../../../game_items/ItemGenerator.js');
const LightTorsoe = require('../../../game_items/equipment/torsoe/LightTorsoe.js');
const Staff = require('../../../game_items/equipment/weapon/Staff.js');
const getMobEquipmenAssignmentFromBags = require('./getMobEquipmenAssignmentFromBags');

const ratling_priest = (level) => {
  const handle = 'ratling_priest';
  const AI_PRIORITIES = ['support', 'dps_close', 'tank', 'interference', 'dps_ranged'];

  const bags = [{
    items: [
      ItemGenerator.getRandomItem(LightTorsoe, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(Staff, 0, Math.min(1,0.1+level/100)),
    ],
  }];

  return {
    handle,
    level,
    getAbilityTrees: ((bound_level) => ([{
      handle: 'ratling_priest_abilities',
      abilities: {
        move: { handle: 'move', level: 1 },
        melee_attack: { handle: 'melee_attack', level: 1 },
        quickness: { handle: 'quickness', level: Math.min(10,Math.round(1+bound_level/2)) },
        healing_prayer: { handle: 'healing_prayer', level: Math.min(10,Math.round(1+bound_level/2)) },
      },
    }])).bind(null, level),
    getBaseAttributes: ((bound_level) => ({
      learning: Math.round(1.4 + bound_level*0.75 + Math.random()*3),
      perception: Math.round(1.0 + bound_level*0.5 + Math.random()*3),
      mana: Math.round(1.3 + bound_level*0.6 + Math.random()*3),
      toughness: Math.round(0.5 + bound_level*0.25 + Math.random()*3),
      strength: Math.round(0.7 + bound_level*0.4 + Math.random()*3),
      willpower: Math.round(1.2 + bound_level*0.6 + Math.random()*3),
      dexterity: Math.round(1.5 + bound_level*0.7 + Math.random()*3),
    })).bind(null, level),
    getDefaultAIConfig: () => AI_PRIORITIES,
    xpValue: level * 275,
    bags,
    build: {
      ai: {
        priorities: AI_PRIORITIES,
        settings: {
          chanceToSkipRoles: true,
          autoMove: true,
          waitForAPMP: false,
        },
      },
      ai_unit: true,
      beast: true,
      equipment_assignment: getMobEquipmenAssignmentFromBags(bags),
      equipped_abilities: ['move', 'melee_attack', 'quickness', 'healing_prayer'],
      handle,
      humanoid: true,
      level,
      team: 'black',
      undead: false,
      unit: true,
    },
  };
};
module.exports = ratling_priest;
