const ItemGenerator = require('../../../game_items/ItemGenerator.js');
const HeavyHelmet = require('../../../game_items/equipment/helmet/HeavyHelmet.js');
const HeavyTorsoe = require('../../../game_items/equipment/torsoe/HeavyTorsoe.js');
const HeavyBoots = require('../../../game_items/equipment/boots/HeavyBoots.js');
const Dagger = require('../../../game_items/equipment/weapon/Dagger.js');
const Shield = require('../../../game_items/equipment/off_hand/Shield.js');
const getMobEquipmenAssignmentFromBags = require('./getMobEquipmenAssignmentFromBags');

const ratling_warrior = (level) => {
  const bags = [{
    items: [
      ItemGenerator.getRandomItem(HeavyHelmet, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(HeavyTorsoe, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(HeavyBoots, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(Dagger, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(Shield, 0, Math.min(1,0.1+level/100)),
    ],
  }];

  return {
    handle: 'ratling_warrior',
    level,
    getAbilityTrees: ((bound_level) => ([{
      handle: 'ratling_warrior_abilities',
      abilities: {
        move: { handle: 'move', level: 1 },
        melee_attack: { handle: 'melee_attack', level: 1 },
        quickness: { handle: 'quickness', level: Math.min(10,Math.round(1+bound_level/2)) },
        shield_block: { handle: 'shield_block', level: Math.min(10,Math.round(1+bound_level/2)) },
      },
    }])).bind(null, level),
    getBaseAttributes: ((bound_level) => ({
      learning: Math.round(1.1 + bound_level*0.45 + Math.random()*3),
      perception: Math.round(1.1 + bound_level*0.65 + Math.random()*3),
      mana: Math.round(1.1 + bound_level*0.35 + Math.random()*3),
      toughness: Math.round(0.7 + bound_level*0.3 + Math.random()*3),
      strength: Math.round(1.0 + bound_level*0.5 + Math.random()*3),
      willpower: Math.round(1.1 + bound_level*0.4 + Math.random()*3),
      dexterity: Math.round(1.6 + bound_level*0.9 + Math.random()*3),
    })).bind(null, level),
    getDefaultAIConfig: () => (['dps_close', 'tank', 'interference', 'support', 'dps_ranged']),
    xpValue: level * 200,
    bags,
    build: {
      ai: {
        priorities: ['dps_close', 'tank', 'interference', 'support', 'dps_ranged'],
        settings: {
          chanceToSkipRoles: true,
          autoMove: true,
          waitForAPMP: false,
        },
      },
      ai_unit: true,
      beast: true,
      equipment_assignment: getMobEquipmenAssignmentFromBags(bags),
      equipped_abilities: ['move', 'melee_attack', 'quickness', 'shield_block'],
      handle: 'ratling_warrior',
      humanoid: true,
      level,
      team: 'black',
      undead: false,
      unit: true,
    },
  };
};
module.exports = ratling_warrior;
