'use strict';

const { MongoClient, ServerApiVersion } = require('mongodb');

// Connection URL
const {
  MONGO_URL = 'mongodb://localhost:27017',
  MONGO_DB_NAME = 'dt-db-dev',
  MONGO_USER,
  MONGO_PWD,
} = {
  ...process.env,
  ...require('dotenv').config().parsed,
};

const mongo_connect_options = {
  auth: {
    username: MONGO_USER,
    password: MONGO_PWD,
  },
  ssl: true,
  tls: true,
  tlsInsecure: true,
  serverApi: {
    version: ServerApiVersion.v1,
    strict: true,
    deprecationErrors: true,
  },
};

// Use connect method to connect to the server
// NOTE: there's no try/catch in this func, because it's used in admin scripts that don't support the logger.
// so, call sites should wrap it.
// this comment is probably a cry for help
let mongodbClient = null;
let db = null;
const getMongoDBClient = async () => {
  if (mongodbClient) {
    return { mongodbClient, db };
  }

  mongodbClient = new MongoClient(MONGO_URL, mongo_connect_options);
  await mongodbClient.connect();
  await mongodbClient.db(MONGO_DB_NAME).command({ ping: 1 });
  console.info(`Connected successfully to MongoDB server at ${MONGO_URL}`); // eslint-disable-line no-console
  db = mongodbClient.db(MONGO_DB_NAME);

  return { mongodbClient, db };
};

module.exports = {
  getMongoDBClient,
};
